﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;


// 自機
public class Fighter : MonoBehaviour {
	float shotSpan = 3f / 60f; // 射撃間隔（sec.）
	float shotWaitTime = 0f;   // 射撃待ち時間（sec.）

	// 最大同時発射可能数
	int BULLET_MAX = 100;
	// 待機リストと稼働リスト
	LinkedList<Bullet> reserveList, activeList;
	void Start() {
		reserveList = new LinkedList<Bullet>();
		activeList = new LinkedList<Bullet>();
		string path = "bullet";
		GameObject pr;
		pr = Resources.Load<GameObject>(path);
		// 事前にオブジェクトの複製を済ませておく
		for (int i = 0; i < BULLET_MAX; ++i) {
			GameObject obj = Object.Instantiate(pr);
			obj.SetActive(false);   // 初回非表示
			Bullet bl = obj.GetComponent<Bullet>();
			// 待機リストへの追加
			reserveList.AddLast(bl);
		}
	}
	// 毎フレームの更新
	void Update() {
		float elapsedTime = Time.deltaTime;

		// Unity Editor上とPCビルドで有効になるシンボル
#if UNITY_EDITOR || UNITY_STANDALONE
		// 左ボタン（0番）を"押している"際の処理
		if (Input.GetMouseButton(0)) {
			Vector3 pos = Input.mousePosition;
			pos = GetScreen2WorldPosition(pos);
			transform.position = pos;
		}
		// 右ボタン（1番）を"押した"際の処理
		// ※今までのInput.GetMouseButtonDown(1)から差し替え
		if (Input.GetMouseButtonDown(1)) {
			shotWaitTime = elapsedTime;
		}
		// 右ボタン（1番）を"押している"際の処理
		if (Input.GetMouseButton(1)) {
			// 一定時間ごとに射撃させる
			shotWaitTime -= elapsedTime;
			if (shotWaitTime < 0f) {
				for (int i = 0; i < 3; ++i) {
					if (this.reserveList.Count == 0)
						break;
					// 待機リストから稼働リストへ追加
					Bullet bl = reserveList.First.Value;
					activeList.AddLast(bl);
					reserveList.RemoveFirst();
					// 射撃
					float angle = -10 + 10 * i;
					Quaternion rot;
					rot = Quaternion.Euler(90f, angle, 0f);
					Vector3 pos = transform.position;
					bl.Shoot(pos, rot, 20);
				}

				// 次の射撃までの時間の付与
				shotWaitTime += shotSpan;
			}
		}
#else
		if (Input.touchCount > 0) {
			// タッチ入力が存在する場合の0番の座標を取得
			Touch touch = Input.GetTouch(0);
			Vector3 pos = touch.position;
			pos = GetScreen2WorldPosition(pos);
			transform.position = pos;
		}
#endif

		// 稼働リストから終了した弾丸を待機リストへ
		LinkedListNode<Bullet> node = activeList.First;
		LinkedListNode<Bullet> prevNode = null;
		int index = 0;  // 稼働インデックス
		while (node != null) {
			Bullet bl = node.Value;
			bool act = bl.Run(index, elapsedTime);
			prevNode = node;
			node = node.Next;
			if (!act) {
				reserveList.AddLast(bl);
				activeList.Remove(prevNode);
				bl.Vanish();
			}
			// インデックスの更新
			++index;
		}
	}

	// スクリーン座標からワールド座標を求める
	// position : 入力スクリーン座標
	// return : 入力地点のワールド座標
	Vector3 GetScreen2WorldPosition(Vector3 position) {
		Camera cam = Camera.main;
		position.z = cam.transform.position.y;
		return cam.ScreenToWorldPoint(position);
	}
}
