﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;


// ボス
public class Boss : MonoBehaviour {
	float shotWaitTime = 0f;// 射撃待ち時間（sec.）
	int shotCount = 0;      // 射撃カウンタ
	int BULLET_MAX = 500;   // 最大同時発射可能数
	LinkedList<Bullet> reserveList, activeList;
	void Start() {
		// 自機と同じ処理、敵機用の弾丸を読込
		reserveList = new LinkedList<Bullet>();
		activeList = new LinkedList<Bullet>();
		GameObject pr;
		pr = Resources.Load<GameObject>("en_bullet");
		for (int i = 0; i < BULLET_MAX; ++i) {
			GameObject obj = Object.Instantiate(pr);
			obj.SetActive(false);
			Bullet bl = obj.GetComponent<Bullet>();
			reserveList.AddLast(bl);
		}
	}
	void Update() {
		float elapsedTime = Time.deltaTime;
		RapidProc(elapsedTime);

		LinkedListNode<Bullet> node = activeList.First;
		LinkedListNode<Bullet> prevNode = null;
		int index = 0;  // 稼働インデックス
		while (node != null) {
			Bullet bl = node.Value;
			bool act = bl.Run(index, elapsedTime);
			prevNode = node;
			node = node.Next;
			if (!act) {
				reserveList.AddLast(bl);
				activeList.Remove(prevNode);
				bl.Vanish();
			}
			// インデックスの更新
			++index;
		}
	}

	const int WAY_COUNT = 8;           // WAY数
	const float WAY_ANGLE = 360f / WAY_COUNT;// WAY間隔
	float SHOT_SPAN = 12f / 60f; // 連射間隔(sec.)
	float SHOT_SPEED = 5f;       // 弾速(m/sec.)
	float RAPID_ANGLE = 5.5f;    // 連射ごとの回転角(deg.)
	Quaternion WAY_ROT = Quaternion.Euler(0f, WAY_ANGLE, 0f);

	// 連射処理
	void RapidProc(float elapsedTime) {
		shotWaitTime -= elapsedTime;
		if (shotWaitTime > 0f)
			return;

		Transform tr = transform;
		Bullet bl = null;
		float angle = RAPID_ANGLE * shotCount;
		Quaternion rot;
		// 時計回り
		rot = Quaternion.Euler(90f, angle, 0f);
		for (int i = 0; i < WAY_COUNT; ++i) {
			// 弾の取り出し
			bl = PickupBullet();
			if (bl == null)
				break;
			bl.Shoot(tr.position, rot, SHOT_SPEED);
			// 次の方向へ更新
			rot = WAY_ROT * rot;
		}
		// 反時計回り
		rot = Quaternion.Euler(90f, -angle, 0f);
		for (int i = 0; i < WAY_COUNT; ++i) {
			bl = PickupBullet();
			if (bl == null)
				break;
			bl.Shoot(tr.position, rot, SHOT_SPEED);
			rot = WAY_ROT * rot;
		}

		++shotCount;
		shotWaitTime += SHOT_SPAN;
	}

	// 待機中の弾丸の取り出し
	Bullet PickupBullet() {
		if (reserveList.Count == 0)
			return null;

		Bullet bl = reserveList.First.Value;
		activeList.AddLast(bl);
		reserveList.RemoveFirst();

		return bl;
	}
	
	public int endurance = 500;
	public GameObject explosion = null; // 爆発Prefab

	void OnTriggerEnter(Collider other) {
		--this.endurance;
		if (this.endurance > 0)
			return;

		Object.Destroy(this.gameObject);
		Vector3 pos = transform.position;
		Quaternion rot = Quaternion.identity;
		Object.Instantiate(explosion, pos, rot);

		// 稼働リストがなくなるまで（すべて）処理する
		while (activeList.Count > 0) {
			// 先頭ノードを処理
			Bullet bl = activeList.First.Value;
			// 弾丸の消滅
			bl.Vanish();
			// 待機リストへ戻す
			reserveList.AddLast(bl);
			// 稼働リストから先頭ノードを切断
			activeList.RemoveFirst();
		}
	}
}
