﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Bullet : MonoBehaviour {
	private bool isRender = false;  // 描画フラグ
	public GameObject explosion = null; // 爆発Prefab

	// 撮影範囲に入っているカメラごとに
	// Updateの前に呼び出されるMonoBehaviourのコールバック
	void OnWillRenderObject() {
		// Gameビューでないカメラ（Unity Editor）からも
		// 呼ばれるのでGameビューのもののみ判定
		Camera cam = Camera.current;
		isRender |= (cam.cameraType == CameraType.Game);
	}

	// 弾丸の実装
	Vector3 direct = Vector3.forward; // 方向
	float speed = 20f;                // 弾速

	// 自前の更新処理　※Update関数は削除する
	// elapsedTime : 経過時間
	// index : 稼働インデックス, time : 経過時間
	public bool Run(int index, float time) {
		Vector3 move = direct * (speed * time);
		transform.Translate(move, Space.World);

		SpriteRenderer rdr = GetComponent<SpriteRenderer>();
		// 描画順序の設定
		rdr.sortingOrder = index;

		bool ret = isRender;
		isRender = false;
		return ret;
	}
	// 外部（今回は自機）から向きと速度を設定
	// pos : 座標, rot : 向き, spd : 速度（m/sec.）
	public void Shoot(Vector3 pos, Quaternion rot, float spd) {
		direct = rot * Vector3.up;
		speed = spd;
		transform.SetPositionAndRotation(pos, rot);

		// 表示有効
		isRender = true;
		gameObject.SetActive(true);
	}
	// 消滅
	public void Vanish() {
		// 表示無効
		isRender = false;
		gameObject.SetActive(false);
	}
	// 方向と速度を指定して射撃
	public void Shoot(Quaternion rot, float spd) {
		direct = rot * Vector3.up;
		speed = spd;
		transform.rotation = rot;
	}
	void OnTriggerEnter() {
		isRender = false;
		gameObject.SetActive(false);
	}
}