﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;


// 敵機
public class Mob : MonoBehaviour {
    // 移動種類の列挙
    public enum MOVE {
        DOWN,       // 画面下
        HOMING,     // 自機追従
        TURN_LEFT,  // 左旋回
        TURN_RIGHT, // 右旋回
    }
    public MOVE moveType = MOVE.DOWN; // 移動種類設定
    float MOVE_SPEED = 3f;  // 移動速度（m/sec.）
    float TURN_SPEED = 60f; // 旋回速度（deg/sec.）
    Vector3 direct = Vector3.back;   // 移動方向
    void Update() {
        float elapsedTime = Time.deltaTime;
        Transform tr = transform;
        Vector3 vup = Vector3.up;
        float turnAngle = TURN_SPEED * elapsedTime;
        Quaternion rot;
        switch (moveType) {
            case MOVE.DOWN:
                direct = Vector3.back;
                break;
            case MOVE.HOMING:
                CalcHomingDirect(elapsedTime);
                break;
            case MOVE.TURN_LEFT:
                // 左（時計回り）旋回
                rot = Quaternion.AngleAxis(turnAngle, vup);
                direct = rot * direct;
                break;
            case MOVE.TURN_RIGHT:
                // 右（反時計回り）旋回
                rot = Quaternion.AngleAxis(-turnAngle, vup);
                direct = rot * direct;
                break;
        }

        // 取得した移動方向へ進行
        Vector3 move = direct * (MOVE_SPEED * elapsedTime);
        Vector3 pos = tr.position + move;
        // 移動方向へ向く回転値の計算
        Vector3 fwd = Vector3.forward;
        rot = Quaternion.FromToRotation(fwd, direct);
        // オブジェクトへ反映
        tr.SetPositionAndRotation(pos, rot);
    }

    // ホーミング処理
    void CalcHomingDirect(float time) {
        Vector3 vup = Vector3.up;
        // 自機オブジェクトをタグから検索
        GameObject pl = GameObject.FindWithTag("Player");
        if (pl == null)
            return;
        Transform tr = transform;
        Transform plTr = pl.transform;
        // 自機方向を取得
        Vector3 target = plTr.position - tr.position;
        target = Vector3.Normalize(target);
        // 現在方向と自機方向の間の角度計算（degree）
        float angle = Vector3.SignedAngle(direct, target, vup);
        float homingAngle = MOVE_SPEED * time;
        Quaternion rot;
        // 旋回角度（deg）
        if (angle < -homingAngle) {
            // 反時計回り
            rot = Quaternion.AngleAxis(-homingAngle, vup);
            direct = rot * direct;
        } else if (angle > homingAngle) {
            // 時計回り
            rot = Quaternion.AngleAxis(homingAngle, vup);
            direct = rot * direct;
        } else {
            // 自機方向を補足
            direct = target;
        }
    }

	public GameObject explosion = null; // 爆発Prefab
    void OnTriggerEnter() {
		Object.Destroy(gameObject);
		// 爆発の生成
		Vector3 pos = transform.position;
		Quaternion rot = Quaternion.identity;
		Object.Instantiate(explosion, pos, rot);
    }
}
